@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,6 +54,7 @@ import java.util.Collections;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
@@ -66,15 +67,20 @@ import com.openexchange.contacts.ldap.exceptions.LdapException;
 import com.openexchange.contacts.ldap.exceptions.LdapException.Code;
 import com.openexchange.contacts.ldap.ldap.LdapGetter;
 import com.openexchange.contacts.ldap.ldap.LdapInterface;
-import com.openexchange.contacts.ldap.ldap.LdapInterface.FillClosure;
 import com.openexchange.contacts.ldap.ldap.LdapJNDIImpl;
+import com.openexchange.contacts.ldap.ldap.LdapInterface.FillClosure;
 import com.openexchange.contacts.ldap.osgi.LDAPServiceRegistry;
 import com.openexchange.contacts.ldap.property.FolderProperties;
+import com.openexchange.contacts.ldap.property.Mappings;
 import com.openexchange.contacts.ldap.property.FolderProperties.ContactTypes;
 import com.openexchange.contacts.ldap.property.FolderProperties.LoginSource;
-import com.openexchange.contacts.ldap.property.Mappings;
+import com.openexchange.contacts.ldap.property.FolderProperties.Sorting;
+import com.openexchange.groupware.AbstractOXException;
 import com.openexchange.groupware.contact.ContactException;
 import com.openexchange.groupware.contact.ContactInterface;
+import com.openexchange.groupware.contact.ContactUnificationState;
+import com.openexchange.groupware.contact.helpers.SpecialAlphanumSortContactComparator;
+import com.openexchange.groupware.contact.helpers.UseCountComparator;
 import com.openexchange.groupware.container.Contact;
 import com.openexchange.groupware.contexts.impl.ContextException;
 import com.openexchange.groupware.contexts.impl.ContextStorage;
@@ -83,20 +89,29 @@ import com.openexchange.groupware.ldap.UserStorage;
 import com.openexchange.groupware.search.ContactSearchObject;
 import com.openexchange.groupware.search.Order;
 import com.openexchange.java.Autoboxing;
+import com.openexchange.search.SearchTerm;
 import com.openexchange.session.Session;
 import com.openexchange.timer.TimerService;
 import com.openexchange.tools.iterator.ArrayIterator;
 import com.openexchange.tools.iterator.SearchIterator;
 
 
+/**
+ * {@link LdapContactInterface}
+ *
+ * @author <a href="mailto:dennis.sieben@open-xchange.com">Dennis Sieben</a>
+ * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+ */
 public class LdapContactInterface implements ContactInterface {
 
+    protected static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(LdapContactInterface.class));
+
     private class ContactLoaderTask implements Runnable {
-        
+
         private final LdapContactInterfaceProvider _contactIFace;
-        
+
         private final Set<Integer> columns;
-        
+
         private final int folderId;
 
         public ContactLoaderTask(final LdapContactInterfaceProvider contactIFace, final int folderId, final Set<Integer> columns) {
@@ -130,49 +145,49 @@ public class LdapContactInterface implements ContactInterface {
         private final int field;
 
         private final Order sort;
-        
+
         /**
          * Initializes a new {@link SortInfo}.
          * @param field
          * @param sort
          */
-        private SortInfo(final int field, final Order sort) {
+        protected SortInfo(final int field, final Order sort) {
             this.field = field;
             this.sort = sort;
         }
 
-        
+
         public final int getField() {
             return field;
         }
 
-        
+
         public final Order getSort() {
             return sort;
         }
-        
+
     }
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(LdapContactInterface.class);
-    
     private static final String MAPPING_TABLE_KEYS = "CONTACT_LDAP_MAPPING_TABLE_KEYS";
 
     private static final String MAPPING_TABLE_VALUES = "CONTACT_LDAP_MAPPING_TABLE_VALUES";
-    
+
     private final int admin_id;
-    
+
     private final String[] attributes;
-    
+
     private final int context;
-    
+
     private final int folderid;
-    
+
     private final FolderProperties folderprop;
-    
+
     private final LdapContactInterfaceProvider contactIFace;
-    
+
     private Session session;
-    
+
+    private Locale locale;
+
     public LdapContactInterface(final int context, final int admin_id, final FolderProperties folderprop, final int folderid, final LdapContactInterfaceProvider contactIFace) {
         this.context = context;
         this.admin_id = admin_id;
@@ -182,8 +197,14 @@ public class LdapContactInterface implements ContactInterface {
         this.attributes = getDistriAttributes(folderprop);
     }
 
+    private Locale getLocale() {
+        if (null == locale) {
+            locale = UserStorage.getStorageUser(session.getUserId(), session.getContextId()).getLocale();
+        }
+        return locale;
+    }
 
-    private static String escapeLDAPSearchFilter(final String ldapfilter) {
+    public static String escapeLDAPSearchFilter(final String ldapfilter) {
         // According to RFC2254 section 4 we escape the following chars so that no LDAP injection can be made:
         // Character       ASCII value
         // ---------------------------
@@ -192,6 +213,8 @@ public class LdapContactInterface implements ContactInterface {
         // )               0x29
         // \               0x5c
         // NUL             0x00
+    	if(ldapfilter == null)
+    		return "";
         final StringBuilder sb = new StringBuilder();
         for (int i = 0; i < ldapfilter.length(); i++) {
             final char curChar = ldapfilter.charAt(i);
@@ -209,8 +232,8 @@ public class LdapContactInterface implements ContactInterface {
                 case ')':
                     sb.append("\\29");
                     break;
-                case '\u0000': 
-                    sb.append("\\00"); 
+                case '\u0000':
+                    sb.append("\\00");
                     break;
                 case '?':
                     sb.append('*');
@@ -237,7 +260,7 @@ public class LdapContactInterface implements ContactInterface {
         throw new LdapException(Code.DELETE_NOT_POSSIBLE);
     }
 
-    public SearchIterator<Contact> getContactsByExtendedSearch(final ContactSearchObject searchobject, final int orderBy, Order order, final int[] cols) throws OXException {
+    public SearchIterator<Contact> getContactsByExtendedSearch(final ContactSearchObject searchobject, final int orderBy, final Order order, final String collation, final int[] cols) throws OXException {
         final Set<Integer> columns = getColumnSet(cols);
         final int folderId;
         {
@@ -254,7 +277,19 @@ public class LdapContactInterface implements ContactInterface {
 
         final Mappings mappings = folderprop.getMappings();
         final ContactTypes contacttype = folderprop.getContacttypes();
-        
+
+        final boolean specialSort;
+        if (orderBy > 0 && orderBy != Contact.SPECIAL_SORTING && orderBy != Contact.USE_COUNT_GLOBAL_FIRST && !Order.NO_ORDER.equals(order)) {
+            specialSort = false;
+        } else {
+            for (final int field : new int[] {
+                Contact.YOMI_LAST_NAME, Contact.SUR_NAME, Contact.YOMI_FIRST_NAME, Contact.GIVEN_NAME, Contact.DISPLAY_NAME,
+                Contact.YOMI_COMPANY, Contact.COMPANY, Contact.EMAIL1, Contact.EMAIL2, Contact.USE_COUNT }) {
+                columns.add(Integer.valueOf(field));
+            }
+            specialSort = true;
+        }
+
         final boolean both = ContactTypes.both.equals(contacttype);
         if (searchobject.isStartLetter()) {
             String userfilter = null;
@@ -288,15 +323,15 @@ public class LdapContactInterface implements ContactInterface {
             }
             arrayList = getLDAPContacts(folderId, columns, getStringFromStringBuilder(user), getStringFromStringBuilder(distri), null, false);
         }
-        
-        sorting(orderBy, order, arrayList);
+
+        sorting(orderBy, order, arrayList, specialSort);
         return new ArrayIterator<Contact>(arrayList.toArray(new Contact[arrayList.size()]));
     }
 
 
     // The all request...
-    public SearchIterator<Contact> getContactsInFolder(final int folderId, final int from, final int to, final int orderBy, final Order order, final int[] cols) throws OXException {
-        
+    public SearchIterator<Contact> getContactsInFolder(final int folderId, final int from, final int to, final int orderBy, final Order order, final String collation, final int[] cols) throws OXException {
+
         final Set<Integer> columns = getColumnSet(cols);
         if (0 == orderBy) {
             columns.add(Integer.valueOf(Contact.SUR_NAME));
@@ -308,7 +343,19 @@ public class LdapContactInterface implements ContactInterface {
         if (0 != orderBy) {
             columns.addAll(getColumnSet(new int[]{orderBy}));
         }
-        
+
+        final boolean specialSort;
+        if (orderBy > 0 && orderBy != Contact.SPECIAL_SORTING && orderBy != Contact.USE_COUNT_GLOBAL_FIRST && !Order.NO_ORDER.equals(order)) {
+            specialSort = false;
+        } else {
+            for (final int field : new int[] {
+                Contact.YOMI_LAST_NAME, Contact.SUR_NAME, Contact.YOMI_FIRST_NAME, Contact.GIVEN_NAME, Contact.DISPLAY_NAME,
+                Contact.YOMI_COMPANY, Contact.COMPANY, Contact.EMAIL1, Contact.EMAIL2, Contact.USE_COUNT }) {
+                columns.add(Integer.valueOf(field));
+            }
+            specialSort = true;
+        }
+
         final List<Contact> arrayList;
         // If a AdminDN is used, all users see the same contacts, so we can cache them...
         if (contentTheSameForAll()) {
@@ -316,7 +363,7 @@ public class LdapContactInterface implements ContactInterface {
             if (null == this.contactIFace.cached_contacts) {
                 try {
                     // Fill
-                    arrayList = getLDAPContacts(folderId, columns, null, null, null, false);
+                    arrayList = getLDAPContacts(folderId, columns, null, null, new SortInfo(Contact.SUR_NAME, Order.ASCENDING), false);
                     this.contactIFace.cached_contacts = Collections.synchronizedList(new ArrayList<Contact>());
                     this.contactIFace.cached_contacts.addAll(arrayList);
                 } finally {
@@ -338,13 +385,13 @@ public class LdapContactInterface implements ContactInterface {
             }
         } else {
             arrayList = getLDAPContacts(folderId, columns, null, null, null, false);
-            
+
         }
-        
+
         // Get only the needed parts...
-        final List<Contact> subList = getSubList(from, to, arrayList);
-        
-        sorting(orderBy, order, subList);
+        final List<Contact> subList = from != 0 || to != 0 ? getSubList(from, to, arrayList) : arrayList;
+
+        sorting(orderBy, order, subList, specialSort);
         final SearchIterator<Contact> searchIterator = new ArrayIterator<Contact>(subList.toArray(new Contact[subList.size()]));
         return searchIterator;
     }
@@ -355,12 +402,12 @@ public class LdapContactInterface implements ContactInterface {
             // Here we start to do some dirty tricks only possible with an AD which stores deleted objects in a special structure
             // this is done for a lifetime of 60 days for forests initially built using W2k and Server 2k3, and 180 days
             // for forests that were initially built with Server 2k3 SP1
-            
+
             // TODO: Check the cols, because not all cols are available on deleted objects
             final Set<Integer> columns = getColumnSet(cols);
             final ArrayList<Contact> contacts = getLDAPContacts(folderId, columns, null, null, null, true);
             removeOlder(since, contacts);
-            
+
             return new ArrayIterator<Contact>(contacts.toArray(new Contact[contacts.size()]));
         } else {
             return EMPTY_ARRAY_ITERATOR;
@@ -395,8 +442,16 @@ public class LdapContactInterface implements ContactInterface {
     }
 
     public Contact getObjectById(final int objectId, final int inFolder) throws OXException {
-        LOG.info("Called getObjectById");
-        return null;
+        final SearchIterator<Contact> objectsById = getObjectsById(new int[][]{new int[]{objectId, inFolder}}, Contact.ALL_COLUMNS);
+        try {
+            if (null != objectsById && objectsById.hasNext()) {
+                return objectsById.next();
+            } else {
+                throw new LdapException(Code.CONTACT_NOT_FOUND, objectId, inFolder);
+            }
+        } catch (final AbstractOXException e) {
+            throw new OXException(e);
+        }
     }
 
     public SearchIterator<Contact> getObjectsById(final int[][] objectIdAndInFolder, final int[] cols) throws OXException {
@@ -405,7 +460,7 @@ public class LdapContactInterface implements ContactInterface {
         for (final int[] object : objectIdAndInFolder) {
             final int object_id = object[0];
             final int folder_id = object[1];
-            
+
             String userfilter = null;
             String distrifilter = null;
             final boolean both = ContactTypes.both.equals(folderprop.getContacttypes());
@@ -425,7 +480,12 @@ public class LdapContactInterface implements ContactInterface {
         LOG.info("Called getUserById");
         return null;
     }
-    
+
+    public Contact[] getUsersById(final int[] userIds, final boolean performReadCheck) throws OXException {
+        LOG.info("Called getUsersById");
+        return null;
+    }
+
     public Contact getUserById(final int userId) throws OXException {
         LOG.info("Called getUserById");
         return null;
@@ -444,17 +504,17 @@ public class LdapContactInterface implements ContactInterface {
         session = s;
         initMappingTable();
     }
-    
+
     public void updateContactObject(final Contact co, final int fid, final Date d) throws OXException, OXConcurrentModificationException, ContactException {
         LOG.info("Called updateContactObject");
     }
 
-    public void updateUserContact(Contact contact, Date lastmodified) throws OXException {
+    public void updateUserContact(final Contact contact, final Date lastmodified) throws OXException {
         LOG.info("Called updateUserContact");
     }
 
     private void addFilterFor(final String fieldname, final String searchString, final StringBuilder sb) {
-        if (null != searchString) {
+        if (null != searchString && null != fieldname) {
             if ("*".equals(searchString)) {
                 sb.append("(");
                 sb.append(fieldname);
@@ -470,7 +530,7 @@ public class LdapContactInterface implements ContactInterface {
     }
 
     private boolean contentTheSameForAll() {
-        return FolderProperties.AuthType.AdminDN.equals(folderprop.getAuthtype());
+        return FolderProperties.AuthType.AdminDN.equals(folderprop.getAuthtype()) || FolderProperties.AuthType.anonymous.equals(folderprop.getAuthtype());
     }
 
 
@@ -489,7 +549,7 @@ public class LdapContactInterface implements ContactInterface {
         final Mappings mappings = folderprop.getMappings();
         final List<String> attr = new ArrayList<String>();
         final String displayname = mappings.getDisplayname();
-        if (0 != displayname.length()) {
+        if (null != displayname && 0 != displayname.length()) {
             attr.add(displayname);
         }
         final String email1 = mappings.getEmail1();
@@ -528,7 +588,7 @@ public class LdapContactInterface implements ContactInterface {
         final boolean both = ContactTypes.both.equals(folderprop.getContacttypes());
         final boolean distributionlist = both || ContactTypes.distributionlists.equals(folderprop.getContacttypes());
         final LdapInterface iface = new LdapJNDIImpl(getLogin(), session.getPassword(), folderprop, deleted, distributionlist, sortField);
-    
+
         try {
             if (both || ContactTypes.users.equals(folderprop.getContacttypes())) {
                 final String filter;
@@ -731,12 +791,103 @@ public class LdapContactInterface implements ContactInterface {
                     arrayList.add(contact);
                 }
             }
-            
+
         });
-        
+
     }
 
-    private void sorting(final int orderBy, final Order order, final List<Contact> subList) {
-        Collections.sort(subList, new ContactComparator(orderBy, order));
+    private void sorting(final int orderBy, final Order order, final List<Contact> subList, final boolean specialSort) {
+        if (Order.NO_ORDER != order && folderprop.getSorting().equals(Sorting.groupware)) {
+            if (orderBy == Contact.USE_COUNT_GLOBAL_FIRST) {
+                java.util.Collections.sort(subList, new UseCountComparator(specialSort, getLocale()));
+                if (Order.DESCENDING.equals(order)) {
+                    java.util.Collections.reverse(subList);
+                }
+            } else if (specialSort) {
+                java.util.Collections.sort(subList, new SpecialAlphanumSortContactComparator(getLocale()));
+                if (Order.DESCENDING.equals(order)) {
+                    java.util.Collections.reverse(subList);
+                }
+            } else {
+                Collections.sort(subList, new ContactComparator(orderBy, order));
+            }
+        } else {
+            // Default sorting
+            Collections.sort(subList, new ContactComparator(-1, order));
+        }
+    }
+
+    public void associateTwoContacts(final Contact master, final Contact slave) throws OXException {
+        throw new UnsupportedOperationException();
+    }
+
+    public List<Contact> getAssociatedContacts(final Contact contact) throws OXException {
+        throw new UnsupportedOperationException();
+    }
+
+    public ContactUnificationState getAssociationBetween(final Contact c1, final Contact c2) throws OXException {
+        throw new UnsupportedOperationException();
+    }
+
+    public Contact getContactByUUID(final String uuid) throws OXException {
+        throw new UnsupportedOperationException();
+    }
+
+    public void separateTwoContacts(final Contact master, final Contact slave) throws OXException {
+        throw new UnsupportedOperationException();
+    }
+
+    public <T> SearchIterator<Contact> getContactsByExtendedSearch(final SearchTerm<T> searchterm, final int orderBy, final Order order, final String collation, final int[] cols) throws OXException {
+        final Mappings mappings = folderprop.getMappings();
+        final ContactTypes contacttype = folderprop.getContacttypes();
+        final boolean both = ContactTypes.both.equals(contacttype);
+        final boolean distributionlistsActive = both || ContactTypes.distributionlists.equals(contacttype);
+
+        ContactSearchtermLdapConverter parser = new ContactSearchtermLdapConverter();
+        parser.setMappings(mappings);
+        parser.setDistributionlistActive(distributionlistsActive);
+        parser.parse(searchterm);
+
+        final Set<Integer> columns = getColumnSet(cols);
+        final int folderId;
+        {
+            final List<String> folders = parser.getFolders();
+            if (null == folders) {
+                throw new LdapException(Code.FOLDERID_OBJECT_NULL);
+            }
+            if (folders.size() != 1) {
+                throw new LdapException(Code.TOO_MANY_FOLDERS);
+            }
+            folderId = Integer.parseInt(folders.get(0));
+        }
+        final ArrayList<Contact> arrayList;
+
+        final boolean specialSort;
+        if (orderBy > 0 && orderBy != Contact.SPECIAL_SORTING && orderBy != Contact.USE_COUNT_GLOBAL_FIRST && !Order.NO_ORDER.equals(order)) {
+            specialSort = false;
+        } else {
+            for (final int field : new int[] {
+                Contact.YOMI_LAST_NAME, Contact.SUR_NAME, Contact.YOMI_FIRST_NAME, Contact.GIVEN_NAME, Contact.DISPLAY_NAME,
+                Contact.YOMI_COMPANY, Contact.COMPANY, Contact.EMAIL1, Contact.EMAIL2, Contact.USE_COUNT }) {
+                columns.add(Integer.valueOf(field));
+            }
+            specialSort = true;
+        }
+
+        String users = null, distributions = null;
+        if (both || ContactTypes.users.equals(contacttype)) {
+            users = parser.getQueryString();
+        }
+        if (distributionlistsActive) {
+            distributions = parser.getQueryString();
+        }
+        if (LOG.isDebugEnabled()) {
+            LOG.debug("Using following filters for users: \"" + users + "\" and distributionlists: \"" + distributions + "\"");
+        }
+        arrayList = getLDAPContacts(folderId, columns, users, distributions, null, false);
+
+        sorting(orderBy, order, arrayList, specialSort);
+        return new ArrayIterator<Contact>(arrayList.toArray(new Contact[arrayList.size()]));
+
     }
 }
