@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -73,25 +73,25 @@ public class LdapException extends OXException {
         DELETE_NOT_POSSIBLE("LDAP contacts cannot be deleted", Category.PERMISSION, 1),
 
         /**
-         * An error occured while trying to read an LDAP attribute: %s
+         * An error occurred while trying to read an LDAP attribute: %s
          */
-        ERROR_GETTING_ATTRIBUTE("An error occured while trying to read an LDAP attribute: %s", Category.INTERNAL_ERROR, 2),
+        ERROR_GETTING_ATTRIBUTE("An error occurred while trying to read an LDAP attribute: %s", Category.INTERNAL_ERROR, 2),
 
         /**
          * The given value "%s" is not possible for a sort field
          */
         SORT_FIELD_NOT_POSSIBLE("The given value \"%s\" is not possible for a sort field", Category.INTERNAL_ERROR, 3),
-    
+
         /**
          * Contacts cannot be inserted in LDAP
          */
         INSERT_NOT_POSSIBLE("Contacts cannot be inserted in LDAP", Category.PERMISSION, 4),
-        
+
         /**
          * The folderid object is null. This is an internal error. Please notify Open-Xchange
          */
         FOLDERID_OBJECT_NULL("The folderid object is null. This is an internal error. Please notify Open-Xchange", Category.INTERNAL_ERROR, 5),
-        
+
         /**
          * The search object contains more than one folder id. This is not supported by this implementation
          */
@@ -116,7 +116,7 @@ public class LdapException extends OXException {
          * Error while trying to create connection to LDAP server: %s
          */
         INITIAL_LDAP_ERROR("Error while trying to create connection to LDAP server: %s", Category.CODE_ERROR, 10),
-        
+
         /**
          * The LDAP search for the user contains too many results
          */
@@ -141,12 +141,12 @@ public class LdapException extends OXException {
          * The imap login for user "%s" is null
          */
         IMAP_LOGIN_NULL("The imap login for user \"%s\" is null", Category.USER_CONFIGURATION, 15),
-        
+
         /**
          * The primary mail for user "%s" is null
          */
         PRIMARY_MAIL_NULL("The primary mail for user \"%s\" is null", Category.USER_CONFIGURATION, 16),
-        
+
         /**
          * The mail address "%s" for distributionentry is invalid
          */
@@ -161,7 +161,7 @@ public class LdapException extends OXException {
          * The attribute "%s" is missing for object "%s"
          */
         MISSING_ATTRIBUTE("The attribute \"%s\" is missing for object \"%s\"", Category.USER_CONFIGURATION, 19),
-        
+
         /**
          * No values mapping table found
          */
@@ -175,25 +175,31 @@ public class LdapException extends OXException {
         /**
          * An error occurred while trying to get the defaultNamingContext attribute
          */
-        ERROR_GETTING_DEFAULT_NAMING_CONTEXT("An error occurred while trying to get the defaultNamingContext attribute", Category.CODE_ERROR, 22);
-
-
+        ERROR_GETTING_DEFAULT_NAMING_CONTEXT("An error occurred while trying to get the defaultNamingContext attribute", Category.CODE_ERROR, 22),
         
+        /**
+         * The contact object with id "%s" could not be found in folder "%s"
+         */
+        CONTACT_NOT_FOUND("The contact object with id \"%s\" could not be found in folder \"%s\"", Category.CODE_ERROR, 23);
+
+
+
+
         /**
          * Message of the exception.
          */
         private final String message;
-    
+
         /**
          * Category of the exception.
          */
         private final Category category;
-    
+
         /**
          * Detail number of the exception.
          */
         private final int number;
-    
+
         /**
          * Default constructor.
          * @param message message.
@@ -206,29 +212,29 @@ public class LdapException extends OXException {
             this.category = category;
             this.number = detailNumber;
         }
-    
+
         public Category getCategory() {
             return category;
         }
-    
+
         public String getMessage() {
             return message;
         }
-    
+
         public int getNumber() {
             return number;
         }
     }
 
     /**
-     * 
+     *
      */
     private static final long serialVersionUID = 3828591312217664226L;
 
     public LdapException(final Code code, final Object... messageArgs) {
         this(code, null, messageArgs);
     }
-    
+
     /**
      * Initializes a new exception using the information provided by the code.
      * @param code code for the exception.
@@ -241,7 +247,7 @@ public class LdapException extends OXException {
             null == code.message ? cause.getMessage() : code.message, cause);
         setMessageArgs(messageArgs);
     }
-    
+
     /**
      * Constructor with all parameters.
      * @param component Component.
@@ -259,7 +265,7 @@ public class LdapException extends OXException {
 
     /**
      * Initializes a new exception using the information provides by the cause.
-     * 
+     *
      * @param cause
      *            the cause of the exception.
      */
