@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -54,7 +54,6 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import com.openexchange.authentication.LoginException;
 import com.openexchange.groupware.container.FolderObject;
-import com.openexchange.groupware.contexts.Context;
 import com.openexchange.groupware.contexts.impl.ContextImpl;
 import com.openexchange.login.LoginResult;
 import com.openexchange.login.LoginHandlerService;
@@ -63,12 +62,12 @@ import com.openexchange.session.Session;
 
 /**
  * {@link LdapUserFolderCreator}
- * 
+ *
  * @author <a href="mailto:dennis.sieben@open-xchange.com">Dennis Sieben</a>
  */
 public class LdapUserFolderCreator implements LoginHandlerService {
 
-    private static final Log LOG = LogFactory.getLog(LdapUserFolderCreator.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(LdapUserFolderCreator.class));
 
     /**
      * Initializes a new {@link LdapUserFolderCreator}.
@@ -82,7 +81,7 @@ public class LdapUserFolderCreator implements LoginHandlerService {
         final Session session = login.getSession();
         final int cid = session.getContextId();
 
-        final Context ctx = new ContextImpl(cid);
+        new ContextImpl(cid);
         final FolderObject fo = new FolderObject();
 //        fo.setPermissionsAsArray(new OCLPermission[] { defaultPerm });
         fo.setDefaultFolder(true);
