@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -155,7 +155,7 @@ public class LdapGetterJNDIImpl implements LdapGetter {
 
     public LdapGetter getLdapGetterForDN(final String dn, String[] attributes) throws LdapException {
         try {
-            return new LdapGetterJNDIImpl(context.getAttributes(dn, attributes), context, dn);
+            return new LdapGetterJNDIImpl(context.getAttributes(LdapJNDIImpl.escapeSlashforJNDI(dn), attributes), context, dn);
         } catch (final NamingException e) {
             throw new LdapException(Code.ERROR_GETTING_ATTRIBUTE, "AttributeDN: " + dn + " - " + e.getMessage());
         }
