@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -99,7 +99,7 @@ public class LdapJNDIImpl implements LdapInterface {
      *
      */
     private class DeletedControl implements Control {
-    
+
         /**
          * For serialization
          */
@@ -107,29 +107,35 @@ public class LdapJNDIImpl implements LdapInterface {
         public byte[] getEncodedValue() {
             return new byte[] {};
         }
-    
+
         public String getID() {
             return "1.2.840.113556.1.4.417";
         }
-    
+
         public boolean isCritical() {
             return true;
         }
     }
 
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(LdapJNDIImpl.class);
-    
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(LdapJNDIImpl.class));
+
     private final FolderProperties folderprop;
-    
+
     private final LdapContext context;
-    
+
     private final boolean deleted;
+    
+    private final boolean distributionlist;
+    
+    private final SortInfo sortField;
 
     private static Map<String, String> MAPPINGTABLE_USERNAME_LDAPBIND = new ConcurrentHashMap<String, String>();
-    
+
     public LdapJNDIImpl(final String login, final String password, final FolderProperties folderprop, final boolean deleted, final boolean distributionlist, final SortInfo sortField) throws LdapException {
         this.folderprop = folderprop;
         this.deleted = deleted;
+        this.distributionlist = distributionlist;
+        this.sortField = sortField;
         try {
             this.context = createContext(login, password);
         } catch (final NamingException e1) {
@@ -141,20 +147,10 @@ public class LdapJNDIImpl implements LdapInterface {
 //      if (!pagedResultControlSupported) {
 //          System.out.println("Paged results are not supported");
 //      }
-
-        try {
-            this.context.setRequestControls(getControls(sortField, distributionlist, folderprop.getPagesize()));
-        } catch (final NamingException e) {
-            LOG.error(e.getMessage(), e);
-            throw new LdapException(Code.ERROR_GETTING_ATTRIBUTE, e.getMessage());
-        } catch (final IOException e) {
-            LOG.error(e.getMessage(), e);
-            throw new LdapException(Code.ERROR_GETTING_ATTRIBUTE, e.getMessage());
-        }
-        
     }
-    
+
     public void search(final String baseDN, final String filter, final boolean distributionslist, final Set<Integer> columns, final FillClosure closure) throws LdapException {
+        initControls(folderprop, distributionlist, sortField);
         final String defaultNamingContext;
         if (deleted) {
             defaultNamingContext = getDefaultNamingContext(context);
@@ -172,13 +168,13 @@ public class LdapJNDIImpl implements LdapInterface {
         final String ownBaseDN;
         final String ownFilter;
         if (deleted) {
-            ownBaseDN = defaultNamingContext;
+            ownBaseDN = escapeSlashforJNDI(defaultNamingContext);
             ownFilter = "(&" + filter + "(isDeleted=TRUE))";
         } else {
-            ownBaseDN = baseDN;
+            ownBaseDN = escapeSlashforJNDI(baseDN);
             ownFilter = filter;
         }
-        
+
         try {
             byte[] cookie = null;
             // Attention: It is important that we progress the objects in a second step after the search
@@ -191,7 +187,7 @@ public class LdapJNDIImpl implements LdapInterface {
                     final Attributes attributes = next.getAttributes();
                     ldapGetterList.add(getLdapGetter(attributes, context, next.getNameInNamespace()));
                 }
-                // Examine the paged results control response 
+                // Examine the paged results control response
                 final Control[] controls = context.getResponseControls();
                 if (controls != null) {
                     for (int i = 0; i < controls.length; i++) {
@@ -207,13 +203,13 @@ public class LdapJNDIImpl implements LdapInterface {
                     context.setRequestControls(new Control[] { new PagedResultsControl(pagesize, cookie, Control.CRITICAL) });
                 }
             } while (null != cookie);
-            
+
             context.setRequestControls(null);
             for (final LdapGetter ldapGetter : ldapGetterList) {
                 try {
                     closure.execute(ldapGetter);
                 } catch (final LdapException e) {
-                    LOG.error("Error occured on distributionlist " + ldapGetter.getObjectFullName());
+                    LOG.error("Error occurred on distributionlist " + ldapGetter.getObjectFullName());
                     throw e;
                 }
             }
@@ -227,7 +223,7 @@ public class LdapJNDIImpl implements LdapInterface {
         }
 
     }
-    
+
     private Control[] getControls(final SortInfo sortField, final boolean distributionlist, final int pagesize) throws IOException, NamingException {
         if (this.deleted) {
             if (0 == pagesize) {
@@ -236,13 +232,15 @@ public class LdapJNDIImpl implements LdapInterface {
                 return new Control[] { new PagedResultsControl(pagesize, Control.CRITICAL), new DeletedControl() };
             }
         } else {
-            if (null != sortField && folderprop.getSorting().equals(Sorting.server)) {
-                final SortKey sortKey = new SortKey(getFieldFromColumn(sortField.getField(), distributionlist), sortField.getSort().equals(Order.ASCENDING), null);
+            final String fieldFromColumn;
+            if (null != sortField && folderprop.getSorting().equals(Sorting.server)&& null != (fieldFromColumn = getFieldFromColumn(sortField.getField(), distributionlist))) {
+                final SortKey sortKey = new SortKey(fieldFromColumn, sortField.getSort().equals(Order.ASCENDING), null);
                 final SortKey[] sortKeyArray = new SortKey[] { sortKey };
                 if (0 == pagesize) {
                     return new Control[] { new SortControl(sortKeyArray, Control.CRITICAL) };
                 } else {
-                    return new Control[] { new SortControl(sortKeyArray, Control.CRITICAL), new PagedResultsControl(pagesize, Control.CRITICAL) };
+                    return new Control[] {
+                        new SortControl(sortKeyArray, Control.CRITICAL), new PagedResultsControl(pagesize, Control.CRITICAL) };
                 }
             } else {
                 if (0 == pagesize) {
@@ -511,7 +509,7 @@ public class LdapJNDIImpl implements LdapInterface {
             searchCtls2.setReturningAttributes(new String[]{"defaultnamingcontext"});
             final NamingEnumeration<SearchResult> search = ctx.search("", "(objectclass=*)", searchCtls2);
             while (search.hasMoreElements()) {
-                SearchResult sr = (SearchResult)search.next();
+                SearchResult sr = search.next();
                 Attributes attrs = sr.getAttributes();
                 if (attrs != null) {
                     for (NamingEnumeration<?> ae = attrs.getAll();ae.hasMoreElements();) {
@@ -525,7 +523,7 @@ public class LdapJNDIImpl implements LdapInterface {
                             }
                         }
                     }
-                } 
+                }
             }
             throw new LdapException(Code.ERROR_GETTING_DEFAULT_NAMING_CONTEXT);
         } catch (final NamingException e) {
@@ -572,6 +570,13 @@ public class LdapJNDIImpl implements LdapInterface {
         return retval;
     }
 
+    // public to be able to test this
+    public static String escapeSlashforJNDI(String baseDN) {
+        // According to http://download.oracle.com/javase/tutorial/jndi/ldap/jndi.html the slash in a dn name has a special meaning to
+        // JNDI. Thus we need to escape it
+        return null != baseDN ? baseDN.replaceAll("/", "\\\\/") : null;
+    }
+
     public static int getSearchControl(final SearchScope searchScope) {
         switch (searchScope) {
         case one:
@@ -621,7 +626,7 @@ public class LdapJNDIImpl implements LdapInterface {
         if (-1 != pooltimeout) {
             env.put("com.sun.jndi.ldap.connect.pool.timeout", String.valueOf(pooltimeout));
         }
-        
+
         return env;
     }
 
@@ -665,6 +670,18 @@ public class LdapJNDIImpl implements LdapInterface {
         }
     }
 
+    private void initControls(final FolderProperties folderprop, final boolean distributionlist, final SortInfo sortField) throws LdapException {
+        try {
+            this.context.setRequestControls(getControls(sortField, distributionlist, folderprop.getPagesize()));
+        } catch (final NamingException e) {
+            LOG.error(e.getMessage(), e);
+            throw new LdapException(Code.ERROR_GETTING_ATTRIBUTE, e.getMessage());
+        } catch (final IOException e) {
+            LOG.error(e.getMessage(), e);
+            throw new LdapException(Code.ERROR_GETTING_ATTRIBUTE, e.getMessage());
+        }
+    }
+
     private String setUpFilter(final String username) {
         return "(&" + folderprop.getUserSearchFilter() + "(" + folderprop.getUserSearchAttribute() + "=" + username + "))";
     }
